######################################################################
#:CQCC::IOPtkWidget
#
# Licensed Materials - Property of IBM
# Rational ClearCase
# (C) Copyright IBM Corp. 2001, 2008 All Rights Reserved
# US Government Users Restricted Rights
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# Copyright 2001 Rational Software Corporation. All Rights Reserved
# This software contains proprietary and confidential information of
# Rational and its suppliers.  Use, disclosure or reproduction is
# prohibited without the prior express written consent of Rational.

=head1 NAME 

CQCC::IOPtkWidget - Base class for IOPtk widgets supporting Perl/Tk

=head1 CLASS DESCRIPTION

This class provides common utility functions for the IOPtk component
widgets supporting the IOPerlTK user interface class. It provides
basic support for initialization and destruction, common attribute
handling, event callbacks, and named object management and lookup.

The IOPerlTK class provides a front end to creating and managing
IOPtkWidget objects so that the Trigger classes do not need to
directly use this class or its subclasses.

SUPPORT POLICY: See TriggerCQCC.pm for the Rational Support Policy. 

=head1 METHODS

=cut

######################################################################
#::Package declarations
######################################################################

require 5.006; 

package CQCC::IOPtkWidget;
use Exporter;
@ISA = ('Exporter');
@EXPORT = qw(&WidgetLookup);

use Tk 800.022;
use strict;

use CQCC::Parm;
use CQCC::MsgCat;
use CQCC::IOBase;
use CQCC::IOPerlTK;

sub Version { return '2.1' };

######################################################################
#::Class Globals 
######################################################################

my %PtkWidgetList = ();       # Hash holding defined IOPtkWidgets

######################################################################
#::method DESTROY

=over 6

=item C<DESTROY()>

This method handles basic destruction of the object, removing it 
from its parent's children lists, destroying the Perl/Tk widgets,
and deleting it from the named lookup tables.

=back

=cut


######################################################################

sub DESTROY {
    my ($self) = @_;

    # Remove self from Parent's list
    if ($self->{PARENTPTR}) {
        $self->{PARENTPTR}->ManageChildren("DELETE", $self);
    }

    # Destroy the top level which will in turn destroy dependents
    my $topWidget = $self->TKSubWidget("top");    
    if ($topWidget) {
        $topWidget->destroy();
    } 

    # Remove from named lookup list
    delete $PtkWidgetList{$self->{NAME}};
}

######################################################################
#::method DoCallback

=over 6

=item C<DoCallback(itemName, itemCmd)>

This method processes a callback event associated with the 
object, usually to support buttons or menu items. 

If the object has a CALLBACK_OBJECT attribute, the itemCmd is
interpreted as a method call to that object with optional
arguments. itemCmd can be either an array reference to provide
arguments or as a string name with the name of the method.

If the object has no CALLBACK_OBJECT attribute, itemCmd is 
evaluated directly. 

=back

=cut

######################################################################

sub DoCallback {
    my ($self, $itemName, $itemCmd) = @_;
    my $callback = $self->{CALLBACK_OBJECT};
    if ($callback) {
        if (ref($itemCmd)) {
            my ($method,@args) = @{ $itemCmd };
            return $callback->$method(@args);
        }
        else {
            return $callback->$itemCmd();
        }
    }
    my $result = eval $itemCmd;
    return $result;
}

######################################################################
#::method new

=over 6

=item C<new CQCC::IOPtkWidget(name, parent, type, key-value pairs)>

This method initializes the object's attributes, adds it to a named
lookup table, adds it as a new child to its parent widget, and 
processes common attribute settings. 

All IOPtk* widgets should be created through the appropriate 
constructor functions in CQCC::IOPerlTK and not through direct calls
to new().

=back

=cut

######################################################################

sub new { 
    my $proto = shift;
    my $class = ref($proto) || $proto;  # Object or class name

    ########################################
    # Process Arguments

    my $self;

    $self->{CLASSNAME} = "CQCC::IOPtkWidget";

    # Pop off the standard arguments: Name, Parent, Type

    $self->{NAME} = shift @_;
    $self->{PARENT} = shift @_;
    $self->{TYPE} = shift @_;

    # Process remaining arguments as KEY=>VALUE pairs and store 

    my @args = @_;
    while (@args) {
        my $key = shift @args;
        $self->{$key} = shift @args;
    } 
    bless($self, $class);

    ########################################
    # Store reference to self in global list
    if (defined $PtkWidgetList{$self->{NAME}}) { 
        &IOMessage("ERROR", [ "IOPTK_E_DUP_WIDGET", $self->{NAME},
                            $self->{PARENT}]);
        return;
    } 
    else {
        $PtkWidgetList{$self->{NAME}} = $self;
    }

    ########################################
    # Store reference to self in parent's children list

    $self->{MSGCAT} = &IOstd()->{MSGCAT};
    $self->{TITLEMSG} = "";
    if ($self->{TITLE}) {
        $self->{TITLEMSG} = $self->{MSGCAT}->Message($self->{TITLE});
    }

    # Only here to cover for root panel (MainWindow)
    if ($self->{PARENT} eq "" ) { 
        $self->{PARENTPTR} = undef;
    } else {
        $self->{PARENTPTR} = $PtkWidgetList{$self->{PARENT}};
        $self->{PARENT_WIDGET} = $self->{PARENTPTR}->{WIDGET};
        if (! defined $self->{PARENTPTR}) {
            &IOMessage("ERROR", ["IOPTK_E_UNKNOWN_PARENT",
                                $self->{PARENT}, $self->{NAME}]);
            return;
        }
        elsif (! $self->{PARENTPTR}->can("ManageChildren")) {
            &IOMessage("ERROR", ["IOPTK_E_NOT_PARENT",$self->{PARENT},
                                $self->{NAME}]);
            return;
        }
        else {
            $self->{PARENTPTR}->ManageChildren("ADD",$self);
        }
        # May be needed later for I18N support
        if ($self->{FONT}) {
            $self->{FONTPTR} = 
                $self->{PARENT_WIDGET}->fontCreate(@{$self->{FONT}});
        }
    }
    return $self;
}

######################################################################
#::method NewSubType

=over 6

=item C<newSubType()>

The IOPtkWidget subclasses can be specialized by a subtype method
to implement a particular widget such as a label or button. This
internal method is called by the subclass to complete initialization 
by calling the subtype method.

=back

=cut

######################################################################

sub NewSubType { 
   my ($self) = @_;
   my $type = $self->{TYPE};
   if ($self->can($type)) {
       $self->$type();
   }
   else {
       &IOMessage("FATAL", 
                  ["IOPTK_E_UNKNOWN_SUBTYPE", $type, 
                   $self->{CLASSNAME}]);
   }
   return $self;
}

######################################################################
#::method PackArgs

=over 6

=item C<@list = PackArgs( {default pack arguments} )>

This internal method is called by a subclass to return a list
arguments to the PerlTK pack() call, either using the default pack
arguments being passed in or the PACK attribute list (preferred)
provided by the new call.

=back

=cut

######################################################################

sub PackArgs { 
   my ($self,@packArgs) = @_;

   if ($self->{PACK}) {
       @packArgs = @{ $self->{PACK} };
   }
   return @packArgs;
}

######################################################################
#::method TKoptions

=over 6

=item C<TKoptions()>

This internal method retrieves any Perl/Tk initialization options
defined by the TK attribute (a list reference) and the FONTPTR
attribute if one is provided. 

=back

=cut

######################################################################

sub TKoptions { 
    my ($self) = @_;
    my @options; 
    if ($self->{FONTPTR}) {
        push @options, (-font => $self->{FONTPTR});
    }
    if ($self->{TK}) {
        push @options, @{ $self->{TK}};
    }
    return @options;
}

######################################################################
#::method TKSubWidget

=over 6

=item C<TKSubWidget(subwidgetName)>

This method retrieves Perl/Tk widgets used in the implementation
of the IOPtkWidget. It is provided to allow more direct access to 
Perl/TK when required but should be used sparingly. 

Subwidget names vary depending on the particular IOPtkWidget class.
This method also supports "TOP" to get the outermost widget used
in the IOPtkWidget implementation.

=back

=cut

######################################################################

sub TKSubWidget { 
    my ($self,$subWidgetName) = @_;
    if ($subWidgetName eq "TOP") {
        return $self->{FRAME} ? $self->{FRAME} : $self->{WIDGET};
    } 
    elsif ($self->{$subWidgetName}) {
        return $self->{$subWidgetName};
    } 
    return;
}

######################################################################
#::function WidgetLookup

=over 6

=item C<WidgetLookup(name, noreport)>

This function returns a pointer to the IOPtkWidget with the given
name. It permits IOPtkWidgets to be created without needing to 
maintain the resultant pointers. If noreport is true, then no 
error messages are reported for unfound widgets; otherwise, a fatal
error is generated. 

=back

=cut

######################################################################

sub WidgetLookup { 
   my ($name, $noreport) = @_;

   if (! defined $PtkWidgetList{$name} && ! $noreport) { 
       &IOMessage("FATAL", [ "IOPTK_E_UNKNOWN_WIDGET", $name]);
       return;
   } 
   return $PtkWidgetList{$name};
}

######################################################################
#::Tail
######################################################################

1;
